\name{[.dtrace}
\alias{[.dtrace}
\alias{[[.dtrace}
\title{Subsetting of density trace objects}
\description{
These functions perform subsetting of density trace objects.
}
\usage{
x[i]
x[[i]]
}
\arguments{
  \item{x}{An object of class \code{dtrace}.}
  \item{i}{Indexing information}
}
\details{
  The first form of subsetting can be used to produce a \code{dtrace}
  object containing a subset of the original object.

  The second form of subsetting provides access to the internal
  components of and individual density estimate.  The result is
  a list which contains \code{x} and \code{y} components which
  give the actual estimates.
}
\value{
  A subset of the \code{dtrace} object passed as an argument.
}
\author{Ross Ihaka}
\seealso{\code{\link{dtrace}}.}
\examples{
## Estimation and plotting of three densities.
## A demonstration of the formula-based interface.
data(iris)
d <- dtrace(Petal.Width ~ Species, data = iris)
summary(d[1])
plot(d[[2]], type = "l")
}
\keyword{distribution}
\keyword{smooth}
\eof
\name{plot.dtrace}
\alias{plot.dtrace}
\title{Plot a Density Trace Object}
\description{
  This function provides a plot method for
  objects of class \code{dtrace}, such as those created
  by \code{dtrace}.
}
\usage{
\method{plot}{dtrace}(x,
  col = NULL, lty = 1:length(x$density), lwd = NULL,
  main = "", xlab = "", ylab = "Density", log = "",
  add = FALSE, draw.legend = length(x$density) > 1 && !add,
  \dots)
}
\arguments{
  \item{x}{An object of class \code{dtrace}.}
  \item{col}{Colors for the lines.}
  \item{lty}{Line textures (recycled if required).}
  \item{lwd}{Line widths (recycled if required).}
  \item{main}{An overall title for the plot.}
  \item{xlab}{A label for the x axis.}
  \item{ylab}{A label for the y axis.}
  \item{log}{A character string indicating which axes should be shown
    on a log scale.  The default is \code{""}.}
  \item{add}{If true, the density traces are drawn on an existing
    plot, otherwise a new plot is created.}
  \item{draw.legend}{Should a legend be added to the plot?}
  \item{\dots}{Additional arguments which are passed to \code{legend}
    so that the legend can be customized.}
}
\details{
  This function produces a plot of density trace object. The plot
  can show a either single density or several superimposed densities. In
  the latter case, a legend is printed on the plot indicating which
  density is which.  By default, different line textures are used to
  distinguish, the densities but it is also possible to vary colors and
  line thicknesses.

  The plots produced by this function have minimal annotation.  This is
  deliberate because, when several densities are superimposed, it is not
  clear what should be written on the plot.  The density trace
  summary function can be used to obtain information about the density
  extimates which could be written onto the plot.
}
\value{
  The value of the argument \code{x} is returned as the invisible value
  of this function.
}
\author{Ross Ihaka}

\seealso{\code{\link{dtrace}}.}
\examples{
## Estimation and plotting of three densities.
## A demonstration of the formula-based interface.
data(iris)
d <- dtrace(Petal.Width ~ Species, data = iris)
plot(d, lty = 1, col=c("red","green4", "blue"),
     legend = c("Setosa", "Versicolor", "Virginica"),
     main = "The Distribution of Iris Petal Width",
     xlab = "Petal Width (cm)")
}
\keyword{hplot}
\keyword{distribution}
\keyword{smooth}
\eof
\name{print.dtrace}
\alias{print.dtrace}
\title{Printing Density Trace Objects}
\description{
This prints kernel density traces.
}
\usage{
\method{print}{dtrace}(x,
    digits = max(4, getOption("digits") - 4),
    \dots)
}
\arguments{
  
  \item{x}{A specification or the data to be used to produce the density
    traces. This can be a numeric vector, a list of numeric vectors or a
    modelling formula which specifies grouping for a  numeric vector.}
  
  \item{digits}{The number of significant digits to appear in the
    printed summary of \code{dt}.}
  
  \item{\dots}{Additional arguments which are passed to the underlying
    \code{\link{print}} function.}
}
\details{
  This function is used to print the objects created by
  \code{\link{dtrace}}.
}
\value{
  The function returns an invisible copy of its first argument.
}
\author{
  Ross Ihaka.
}
\seealso{
  \code{\link{dtrace}},
  \code{\link{summary.dtrace}},
  \code{\link{print.summary.dtrace}}.
}

\examples{
## Simple density estimation.
data(faithful)
d <- print(dtrace(faithful$eruptions, sd = "sj"))
}
\keyword{distribution}
\keyword{smooth}
\eof
\name{dtrace}
\alias{dtrace}
\alias{dtrace.numeric}
\alias{dtrace.list}
\alias{dtrace.formula}
\title{Density Traces}
\description{
This function computes kernel density trace objects.
Related methods can be used to plot and manpulate this
kind of object.
}
\usage{
dtrace(x,  bw, sd = "nrd0",
    kernel = "gaussian", \dots)
}
\arguments{
  
  \item{x}{A specification or the data to be used to produce the density
    traces. This can be a numeric vector, a list of numeric vectors or a
    modelling formula which specifies grouping for a  numeric vector.}
  
  \item{bw}{The bandwidth of the kernel used to estimate the
    density. This the standard deviation of the kernel divided by
    \eqn{\sqrt{12}}{sqrt(12)}. This measure has been choosen so that the
    bandwith of a rectangular kernel is the width of the kernel. This
    makes direct comparison with histograms possible. If several
    densities are to be computed with a single call to
    \code{dtrace}, a vector of \code{bw} values be
    specified. These are recycled if necessary.}
  
  \item{sd}{The standard deviation of the smoothing kernel. This can be
    used in place of \code{bw} to specify the amount of smoothing which
    is to occur.  This can either be a number, or a character string
    which specifies an automatic method for choosing the standard
    deviation (see \code{\link{bw.nrd}}).  If several densities are
    to be computed with a single call to \code{dtrace}, a vector
    of \code{sd} values be specified. These are recycled if necessary.}

  \item{kernel}{The smoothing kernel to be used.  A list of possible
    kernels is given in \code{\link{density}}.  If several densities are
    to be computed with a single call to \code{dtrace}, a vector
    of \code{kernel} values be specified. These are recycled if necessary.}
  
  \item{\dots}{Additional arguments which are passed to the
    \code{\link{density}} function.}
}
\details{
  This function provides a way to estimate probability density functions
  for one or more samples of observations.  The estimation is carried
  out by using a smoothing kernel, which is convolved with the empirical
  distribution function of the data values to produce a smooth density
  estimate.  A variety of different smoothing kernels are available,
  with the default being \code{gaussian}.

  There are two ways in which to specify the degree of smoothing
  provided by the kernel.  The first is to use the argument \code{sd} to
  specify the kernel standard deviation.  The second is to use the
  argument \code{bw} to specify the \emph{bandwidth} of the kernel.
  The definition of bandwidth is the width of the rectangular kernel
  which has the same standard deviation as the kernel. Specifying a
  rectangular kernel (\code{kern="r"}) means that \code{bw} gives the
  cell-width for a moving cell histogram estimate.
 
}
\value{
  An object of class \code{dtrace}.  Summary, plot and
  subsetting methods exist for this type of object.

  It can be useful to know that the object is implemented as a list with
  components:
  \item{density}{A list containing the individual density estimates}
  \item{xlim}{The \eqn{x}{x} range spanned by the ordinates of the estimates.}
  \item{ylim}{The \eqn{y}{y} range spanned by the coordinates of the estimates.}
  \item{nobs}{The sample size.}
  \item{kernel}{The kernel(s) used.}
  \item{bw}{The kernel bandwidth(s).}
  \item{nobs}{The kernel standard deviation(s).}
}
\references{
  Scott, D. W. (1992)
  \emph{Multivariate Density Estimation. Theory, Practice and Visualization}.
  New York: Wiley.

  Sheather, S. J. and Jones M. C. (1991)
  A reliable data-based bandwidth selection method for kernel density
  estimation.
  \emph{J. Roy. Statist. Soc.} \bold{B}, 683--690.

  Silverman, B. W. (1986)
  \emph{Density Estimation}.
  London: Chapman and Hall.

  Venables, W. N. and Ripley, B. D. (1999)
  \emph{Modern Applied Statistics with S-PLUS}.
  New York: Springer.
}

\author{Ross Ihaka.  The underlying density estimation code was written
  by Ross Ihaka, Martin Maechler and Brian Ripley.}

\note{
  Note that I (Ross Ihaka) take full blame for the use of \code{bw} in
  this function to describe the amount of kernel smoothing.  Although
  this might be non-standard, I find it pedagogically useful, to
  motivate the shift from fixed-cell histograms to moving-cell
  histograms and thence to general kernel density extimates.
  I think it also provides a degree of intuition which helps in
  understanding the size of feature which is likely to be smoothed
  away by the kernel.
}

\seealso{
  \code{\link{density}},
  \code{\link{hist}},
  \code{\link{bw.nrd}}.
  \code{\link{plot.dtrace}},
  \code{\link{print.dtrace}},
  \code{\link{summary.dtrace}},
  \code{\link{print.summary.dtrace}},
  \code{\link{[.dtrace}},
  \code{\link{[[.dtrace}}.
}

\examples{
## The rectangular kernel with bw = 1.
plot(dtrace(0, bw=1, kern="r"),
     main = "Rectangular Kernel (bw = 1)",
     xlab = "This is the output from ``dtrace''")

## Compare this with the density function.
## It's hard to see how bw = 1 here..
plot(density(0, bw=1, kern="r"),
   main = "Rectangular Kernel (bw = 1)",
   xlab = "This is the output from ``density''")

## The Gaussian kernel with sd = 1.
plot(dtrace(0, sd=1, kern="g"),
     main = "Gaussian Kernel (sd = 1)")

## A moving-cell histogram and an equivalent
## fixed-cell histogram.  Note that the fixed-cell
## variant is less stable than it appears.
x <- rnorm(100)
d <- dtrace(x, bw=.5, kern = "r", n=1024)
hist(x, breaks = seq(floor(min(x)), ceiling(max(x)), by = .5),
     prob = TRUE, ylim = d$ylim, border = "gray40",
     main = "Fixed and Moving-Cell Histograms")
plot(d, col = "red", add = TRUE)

## A comparison of rectangular and Gaussian kernels
## with the same bandwidth.
d <- dtrace(list(Rectangular = x,
                       Gaussian = x),
            bw = 1, kern = c("r", "g"))
plot(d, col = c("green4", "red"), lty = 1,
     main = "``Equivalent Bandwidth'' Kernels",
     xlab = "Rectangular and Gaussian, (bw = 1, sd = 0.2887)")

## Simple density estimation.
## Here we are using automatic bandwidth selection.
data(faithful)
d <- dtrace(faithful$eruptions, sd = "sj")
summary(d)
plot(d, xlab = "Eruption Time in Minutes",
     main = "Old Faithful Data")

## A customized plot.  This is just to show how
## to access the underlying density structure.
plot(d$density[[1]], type = "n",
     xlab = "Eruption Time in Minutes",
     ylab = "Density",
     main = "Old Faithful Data")
polygon(d$density[[1]], col = "wheat")

## Estimation and plotting of three densities.
## A demonstration of the formula-based interface.
data(iris)
d <- dtrace(Petal.Width ~ Species, data = iris)
plot(d, lty = 1, col = c("red","green4", "blue"),
     legend = c("Setosa", "Versicolor", "Virginica"),
     main = "The Distribution of Iris Petal Width",
     xlab = "Petal Width (cm)")
}
\keyword{distribution}
\keyword{smooth}
\eof
\name{summary.dtrace}
\alias{summary.dtrace}
\alias{print.summary.dtrace}
\title{Summaries for dtrace Objects}
\description{
  This function gathers and returns useful information from
  density trace objects.  A print method is defined so that the
  values computed by this function are printed nicely.
}
\usage{
\method{summary}{dtrace}(object, \ldots)
\method{print}{summary.dtrace}(x, digits = max(4, getOption("digits") - 4), \dots)
}
\arguments{
  \item{object}{An object of class \code{dtrace}.}
  \item{x}{An object of class \code{summary.dtrace}.}
  \item{digits}{The number of significant digits to be used in
    printing.}
  \item{\dots}{Other arguments to be passed to print.}
}
\value{
  The \code{summary} method for \code{dtrace} objects returns a
  list with components:
  \item{xlim}{The range of x values covered by all the density estimates
    in the first argument.}
  \item{ylim}{The range of y values covered by all the density estimates
    in the first argument.}
  \item{names}{The names associated with each group for which a density
    estimate was obtained.}
  \item{nobs}{The number of observations in each group for which a
    density estimate was obtained.}
  \item{bw}{The bandwidth(s) of the kernel(s) used to compute the
    density estimates (see \code{\link{dtrace}}).}
  \item{sd}{The standard deviation(s) of the kernel(s) used to compute
    the density estimates (see \code{\link{dtrace}}).}
}
\author{Ross Ihaka.}
\seealso{\code{\link{dtrace}}.}
\examples{
## Simple density estimation.
## Here we are using automatic bandwidth selection.
data(faithful)
d <- dtrace(faithful$eruptions, sd = "sj")
plot(d, xlab = paste("Kernel Bandwidth =",
                     signif(summary(d)$bw, 4)),
     main = "Old Faithful Data")
}
\keyword{distribution}
\keyword{smooth}
\eof
